#ifndef __PVS_H
#define __PVS_H

#include "common.h"
#include "Bvh.h"

namespace CHCDemoEngine
{

class BvhNode;
class Bvh;


/** Class representing the Potentially Visible Set (PVS) from a view cell.
*/
class Pvs
{

public:

	//Pvs() {}

	void Clear();

	bool Empty() const;

	void AddEntry(Bvh *bvh, BvhNode *node);

	int GetSize() const;

	inline SceneEntity *GetEntry(int i) const;

	inline void AddEntry(SceneEntity *ent);


protected:

	/// hash table of PVS entries
	SceneEntityContainer mEntries; 
};


inline SceneEntity *Pvs::GetEntry(int i) const 
{
	return mEntries[i]; 
}


inline void Pvs::AddEntry(SceneEntity *ent) 
{
	mEntries.push_back(ent); 
}


inline int Pvs::GetSize() const
{ 
	return (int)mEntries.size(); 
}


inline void Pvs::AddEntry(Bvh *bvh, BvhNode *node) 
{
	int geometrySize;
	SceneEntity **entities;
	entities = bvh->GetGeometry(node, geometrySize);

	for (int i = 0; i < geometrySize; ++ i)
	{
		AddEntry(entities[i]);
	}
}


}

#endif